@@ -1,6 +1,5 @@
 # -*- coding: utf-8 -*-
 '''Wrappers for various units of text.'''
-import re
 import sys
 from collections import Counter
 
@@ -8,39 +7,71 @@ import nltk
 from nltk.tokenize import sent_tokenize, word_tokenize
 from .np_extractor import NPExtractor
 from .decorators import cached_property
+from .utils import lowerstrip, strip_punc
+from .inflect import singularize, pluralize
+from .sentiment import sentiment as _sentiment
+
+
+class WordList(list):
+    '''A list-like collection of words.'''
+    def __init__(self, collection):
+        super(WordList, self).__init__(collection)
+        self._collection = collection
+
+    def __repr__(self):
+        return "WordList({0})".format(repr(self._collection))
+
+    def __getitem__(self, index):
+        '''Returns a string at the given index.'''
+        return self._collection[index]  # Just return a string
+
+    def __getslice__(self, i, j):
+        '''Returns a new WordList with items i:j.'''
+        return self.__class__(list.__getslice__(self, i, j))
+
+    def singularize(self):
+        '''Return the single version of each word in this WordList.'''
+        return [singularize(word) for word in self]
+
+    def pluralize(self):
+        '''Return the plural version of each word in this WordList.'''
+        return [pluralize(word) for word in self]
 
 
 class BaseBlob(object):
     '''An abstract base class that all text.blob classes will inherit from.
-    Includes words, POS tag, NP, and word count properties. Also includes 
+    Includes words, POS tag, NP, and word count properties. Also includes
     basic dunder and string methods for making objects like Python strings.
     '''
-    
-    _REPLACE_REGEX = '(\n|\!|\?|\(|\)|\-|\,|\.|\;|\t|\r)'
-
     def __init__(self, text):
         self.raw = text
-        self.stripped = self._lowerstrip(text)
+        self.stripped = lowerstrip(text)
 
-    @cached_property 
+    @cached_property
     def words(self):
-        '''Returns list of word tokens. These are extracted only once, when
-        this property is first accessed, then it is stored in an instance 
-        variable.
+        '''Returns list of word tokens.
         '''
-        return word_tokenize(self.raw)
+        return WordList([strip_punc(word) for word in word_tokenize(self.raw)
+            if strip_punc(word)])  # Excludes "words" that are just punctuation
+
+    @cached_property
+    def sentiment(self):
+        '''Returns sentiment scores as a tuple of the form
+        `(polarity, subjectivity)` where polarity is in the range [-1.0, 1.0]
+        and subjectivity is in the range [0.0, 1.0].
+        '''
+        return _sentiment(self.raw)
 
     @cached_property
     def noun_phrases(self):
         extractor = NPExtractor(self.raw)
-        return [self._lowerstrip(phrase)
-                                for phrase in extractor.extract()
-                                if len(phrase) > 1]
+        return WordList([lowerstrip(phrase) for phrase in extractor.extract()
+                if len(phrase) > 1])
 
     @cached_property
     def pos_tags(self):
         '''Returns an array of tuples of the form (word, POS tag).
-        
+
         Example:
             [('At', 'IN'), ('eight', 'CD'), ("o'clock", 'JJ'), ('on', 'IN'),
                     ('Thursday', 'NNP'), ('morning', 'NN')]
@@ -53,7 +84,7 @@ class BaseBlob(object):
         '''Dictionary of word frequencies in this text. Internally
         uses collections.Counter.
         '''
-        stripped_words = [self._lowerstrip(word) for word in self.words]
+        stripped_words = [lowerstrip(word) for word in self.words]
         return Counter(stripped_words)
 
     @cached_property
@@ -99,12 +130,12 @@ class BaseBlob(object):
         '''
         if isinstance(index, int):
             return self.raw[index]  # Just return a single character
-        else: 
+        else:
             # Return a new blob object
             return self.__class__(self.raw[index])
 
     def __add__(self, other):
-        '''Concatenates two text objects the same way Python strings are 
+        '''Concatenates two text objects the same way Python strings are
         concatenated.
 
         Arguments:
@@ -131,11 +162,6 @@ class BaseBlob(object):
     def lower(self):
         return self.__class__(self.raw.lower())
 
-    def _lowerstrip(self, text):
-        '''Makes text all lowercase and strips punctuation.'''
-        return re.sub(BaseBlob._REPLACE_REGEX, '', 
-                        text.lower()).strip()
-
 
 class TextBlob(BaseBlob):
     """A general text block, meant for larger bodies of text (esp. those
@@ -165,7 +191,7 @@ class TextBlob(BaseBlob):
         return [sentence.dict for sentence in self.sentences]
 
     def _create_sentence_objects(self, blob):
-        '''Returns a list of Sentence objects given 
+        '''Returns a list of Sentence objects given
         a list of sentence strings. Attempts to handle sentences that
         have more than one puntuation mark at the end of the sentence.
         Examples: "An ellipses is no problem..." or "This is awesome!!!"
@@ -175,19 +201,19 @@ class TextBlob(BaseBlob):
         # if there is only one sentence or string of text
         if len(sentences) <= 1:
             sentence_objects.append(
-                Sentence(sentences[0], start_index=0, 
+                Sentence(sentences[0], start_index=0,
                             end_index=len(sentences[0]) - 1)
             )
-        # If there are many sentences 
-        else:     
+        # If there are many sentences
+        else:
             char_index = 0  # Keeps track of character index within the blob
             for i, raw_sentence in enumerate(sentences):
-                # Compute the start and end indices of the sentence 
+                # Compute the start and end indices of the sentence
                 # within the blob
                 start_index = char_index
                 char_index += len(raw_sentence)
 
-                # Sometimes the NLTK tokenizer misses some punctuation when 
+                # Sometimes the NLTK tokenizer misses some punctuation when
                 # there are multiple punctuations, e.g. with ellipses ("...")
                 # or multiple exclamation points ("!!!")
                 try:
@@ -197,13 +223,13 @@ class TextBlob(BaseBlob):
                     if len(raw_sentence) <= 1:
                         continue
                     pass
-                # If the next token has a length of 1, assume it's a punctuation
-                if len(next) == 1: 
+                # If the next token is 1 character, assume it's a punctuation
+                if len(next) == 1:
                     raw_sentence += next  # append the extra punctuation
-                    char_index +=1  # also correct the char_index
+                    char_index += 1  # also correct the char_index
                 # Create a Sentence object and add it the the list
                 sentence_objects.append(
-                    Sentence(raw_sentence, start_index=start_index, 
+                    Sentence(raw_sentence, start_index=start_index,
                                             end_index=char_index))
         return sentence_objects
 
@@ -217,7 +243,7 @@ class Sentence(BaseBlob):
         - `sentence`: A string, the raw sentence.
         - `start_index`: An int, the index where this sentence begins
                             in a TextBlob. If not given, defaults to 0.
-        - `end_index`: An int, the index where this sentence ends in 
+        - `end_index`: An int, the index where this sentence ends in
                             a TextBlob. If not given, defaults to the
                             length of the sentence - 1.
         '''
@@ -233,6 +259,6 @@ class Sentence(BaseBlob):
             "start_index": self.start_index,
             "end_index": self.end_index,
             "stripped_sentence": self.stripped,
-            "noun_phrases": self.noun_phrases
+            "noun_phrases": self.noun_phrases,
+            "sentiment": self.sentiment
         }
-
